@@ -3,7 +3,7 @@
 #include <stdio.h>
 #include <assert.h>
 
-bool print_model(clingo_model_t *model) {
+bool print_model(clingo_model_t const *model) {
   bool ret = true;
   clingo_symbol_t *atoms = NULL;
   size_t atoms_n;
@@ -65,7 +65,7 @@ out:
 bool solve(clingo_control_t *ctl, clingo_solve_result_bitset_t *result) {
   bool ret = true;
   clingo_solve_handle_t *handle;
-  clingo_model_t *model;
+  clingo_model_t const *model;
 
   // get a solve handle
   if (!clingo_control_solve(ctl, clingo_solve_mode_yield, NULL, 0, NULL, NULL, &handle)) { goto error; }
@@ -138,8 +138,11 @@ int main(int argc, char const **argv) {
   // get the backend
   if (!clingo_control_backend(ctl, &backend)) { goto error; }
 
+  // prepare the backend for adding rules
+  if (!clingo_backend_begin(backend)) { goto error; }
+
   // add an additional atom (called d below)
-  if (!clingo_backend_add_atom(backend, &atom_ids[3])) { goto error; }
+  if (!clingo_backend_add_atom(backend, NULL, &atom_ids[3])) { goto error; }
 
   // add rule: d :- a, b.
   body[0] = atom_ids[0];
@@ -151,6 +154,9 @@ int main(int argc, char const **argv) {
   body[1] = atom_ids[2];
   if (!clingo_backend_rule(backend, false, NULL, 0, body, sizeof(body)/sizeof(*body))) { goto error; }
 
+  // finalize the backend
+  if (!clingo_backend_end(backend)) { goto error; }
+
   // solve
   if (!solve(ctl, &solve_ret)) { goto error; }
 
