@@ -662,6 +662,14 @@ void ASTBuilder::showsig(Location const &loc, Sig sig, bool csp) {
     statement_(loc, clingo_ast_statement_type_show_signature, stm);
 }
 
+void ASTBuilder::defined(Location const &loc, Sig sig) {
+    clingo_ast_defined_t defined;
+    defined.signature = sig.rep();
+    clingo_ast_statement stm;
+    stm.defined = create_(defined);
+    statement_(loc, clingo_ast_statement_type_defined, stm);
+}
+
 void ASTBuilder::show(Location const &loc, TermUid t, BdLitVecUid body, bool csp) {
     auto bd = bodies_.erase(body);
     clingo_ast_show_term_t show;
@@ -1025,6 +1033,9 @@ public:
                 auto &y = *stm.show_signature;
                 return prg_.showsig(parseLocation(stm.location), Sig(y.signature), y.csp);
             }
+            case clingo_ast_statement_type_defined: {
+                return prg_.defined(parseLocation(stm.location), Sig(stm.defined->signature));
+            }
             case clingo_ast_statement_type_show_term: {
                 auto &y = *stm.show_term;
                 return prg_.show(parseLocation(stm.location), parseTerm(y.term), parseBodyLiteralVec(y.body, y.size), y.csp);
