@@ -138,7 +138,9 @@ ClingoControl::ClingoControl(Scripts &scripts, bool clingoMode, Clasp::ClaspFaca
 , pgf_(pgf)
 , psf_(psf)
 , logger_(printer, messageLimit)
-, clingoMode_(clingoMode) { }
+, clingoMode_(clingoMode) {
+    clasp->ctx.output.theory = &theory_;
+}
 
 void ClingoControl::parse() {
     if (!parser_->empty()) {
@@ -151,6 +153,11 @@ void ClingoControl::parse() {
     }
 }
 
+ClingoPropagateInit::ClingoPropagateInit(Control &c, Clasp::ClingoPropagatorInit &p)
+: c_{c}, p_{p}, a_{*static_cast<ClingoControl&>(c).clasp_->ctx.solver(0)} {
+    p_.enableHistory(false);
+}
+
 Potassco::Lit_t ClingoPropagateInit::mapLit(Lit_t lit) {
     const auto& prg = static_cast<Clasp::Asp::LogicProgram&>(*static_cast<ClingoControl&>(c_).clasp_->program());
     return Clasp::encodeLit(prg.getLiteral(lit, Clasp::Asp::MapLit_t::Refined));
@@ -160,6 +167,10 @@ int ClingoPropagateInit::threads() {
     return static_cast<ClingoControl&>(c_).clasp_->ctx.concurrency();
 }
 
+Potassco::AbstractAssignment const &ClingoPropagateInit::assignment() const {
+    return a_;
+}
+
 void ClingoControl::parse(const StringVec& files, const ClingoOptions& opts, Clasp::Asp::LogicProgram* claspOut, bool addStdIn) {
     using namespace Gringo;
     logger_.enable(Warnings::OperationUndefined, !opts.wNoOperationUndefined);
@@ -229,47 +240,60 @@ void ClingoControl::ground(Control::GroundVec const &parts, Context *context) {
     }
     if (!parts.empty()) {
         Ground::Parameters params;
-        for (auto &x : parts) { params.add(x.first, SymVec(x.second)); }
-        auto gPrg = prg_.toGround(out_->data, logger_);
+        std::set<Sig> sigs;
+        for (auto &x : parts) {
+            params.add(x.first, SymVec(x.second));
+            sigs.emplace(x.first, numeric_cast<uint32_t>(x.second.size()), false);
+        }
+        auto gPrg = prg_.toGround(sigs, out_->data, logger_);
         LOG << "*********** intermediate program ***********" << std::endl << gPrg << std::endl;
         LOG << "************* grounded program *************" << std::endl;
         auto exit = onExit([this]{ scripts_.resetContext(); });
         if (context) { scripts_.setContext(*context); }
-        gPrg.ground(params, scripts_, *out_, false, logger_);
+        gPrg.ground(params, scripts_, *out_, logger_);
     }
 }
 
-void ClingoControl::main() {
-    if (scripts_.callable("main")) {
-        incremental_ = true;
+void ClingoControl::main(IClingoApp &app, StringVec const &files, const ClingoOptions& opts, Clasp::Asp::LogicProgram* out) {
+    incremental_ = true;
+    if (app.has_main()) {
+        parse({}, opts, out, false);
         clasp_->enableProgramUpdates();
-        scripts_.main(*this);
-    }
-    else if (incmode_) {
-        incremental_ = true;
-        clasp_->enableProgramUpdates();
-        incmode(*this);
+        app.main(*this, files);
     }
     else {
-        incremental_ = false;
-        claspConfig_.releaseOptions();
-        Control::GroundVec parts;
-        parts.emplace_back("base", SymVec{});
-        ground(parts, nullptr);
-        solve({}, 0, nullptr)->get();
+        parse(files, opts, out);
+        if (scripts_.callable("main")) {
+            clasp_->enableProgramUpdates();
+            scripts_.main(*this);
+        }
+        else if (incmode_) {
+            clasp_->enableProgramUpdates();
+            incmode(*this);
+        }
+        else {
+            incremental_ = false;
+            claspConfig_.releaseOptions();
+            Control::GroundVec parts;
+            parts.emplace_back("base", SymVec{});
+            ground(parts, nullptr);
+            solve({nullptr, 0}, 0, nullptr)->get();
+        }
     }
 }
 bool ClingoControl::onModel(Clasp::Model const &m) {
+    bool ret = true;
     if (eventHandler_) {
+        theory_.reset();
         std::lock_guard<decltype(propLock_)> lock(propLock_);
         ClingoModel model(*this, &m);
-        return eventHandler_->on_model(model);
+        ret = eventHandler_->on_model(model);
     }
-    return true;
+    return ret;
 }
 void ClingoControl::onFinish(Clasp::ClaspFacade::Result ret) {
     if (eventHandler_) {
-        eventHandler_->on_finish(convert(ret));
+        eventHandler_->on_finish(convert(ret), &step_stats_, &accu_stats_);
         eventHandler_ = nullptr;
     }
 }
@@ -342,11 +366,15 @@ unsigned ClingoControl::getRootKey() {
 ConfigProxy &ClingoControl::getConf() {
     return *this;
 }
-USolveFuture ClingoControl::solve(Assumptions &&ass, clingo_solve_mode_bitset_t mode, USolveEventHandler cb) {
-    prepare(std::move(ass));
+USolveFuture ClingoControl::solve(Assumptions ass, clingo_solve_mode_bitset_t mode, USolveEventHandler cb) {
+    prepare(ass);
     if (clingoMode_) {
         static_assert(clingo_solve_mode_yield == static_cast<clingo_solve_mode_bitset_t>(Clasp::SolveMode_t::Yield), "");
         static_assert(clingo_solve_mode_async == static_cast<clingo_solve_mode_bitset_t>(Clasp::SolveMode_t::Async), "");
+        if (cb) {
+            step_stats_.init(statistics(), "user_step");
+            accu_stats_.init(statistics(), "user_accu");
+        }
         eventHandler_ = std::move(cb);
         return gringo_make_unique<ClingoSolveFuture>(*this, static_cast<Clasp::SolveMode_t>(mode));
     }
@@ -360,14 +388,10 @@ void ClingoControl::interrupt() {
 bool ClingoControl::blocked() {
     return clasp_->solving();
 }
-void ClingoControl::prepare(Assumptions &&ass) {
+void ClingoControl::prepare(Assumptions ass) {
     eventHandler_ = nullptr;
     // finalize the program
-    if (update()) {
-        // pass assumptions to the backend
-        out_->assume(std::move(ass));
-        out_->endStep(true, logger_);
-    }
+    if (update()) { out_->endStep(ass); }
     grounded = false;
     if (clingoMode_) {
         Clasp::ProgramBuilder *prg = clasp_->program();
@@ -391,6 +415,22 @@ void *ClingoControl::claspFacade() {
     return clasp_;
 }
 
+const char* TheoryOutput::first(const Clasp::Model&) {
+    index_ = 0;
+    return next();
+}
+
+const char* TheoryOutput::next() {
+    if (index_ < symbols_.size()) {
+        std::stringstream ss;
+        symbols_[index_].print(ss);
+        current_ = ss.str();
+        ++index_;
+        return current_.c_str();
+    }
+    return nullptr;
+}
+
 void ClingoControl::registerPropagator(std::unique_ptr<Propagator> p, bool sequential) {
     propagators_.emplace_back(gringo_make_unique<Clasp::ClingoPropagatorInit>(*p, propLock_.add(sequential)));
     claspConfig_.addConfigurator(propagators_.back().get(), Clasp::Ownership_t::Retain);
@@ -399,7 +439,8 @@ void ClingoControl::registerPropagator(std::unique_ptr<Propagator> p, bool seque
 }
 
 void ClingoControl::cleanupDomains() {
-    out_->endStep(false, logger_);
+    // NOTE: should no longer be necessary because all translation is done after ground()
+    //out_->endGround(logger_);
     if (clingoMode_) {
         Clasp::Asp::LogicProgram &prg = static_cast<Clasp::Asp::LogicProgram&>(*clasp_->program());
         prg.endProgram();
@@ -421,17 +462,18 @@ std::string ClingoControl::str() {
     return "[object:IncrementalControl]";
 }
 
-void ClingoControl::assignExternal(Symbol ext, Potassco::Value_t val) {
+void ClingoControl::assignExternal(Potassco::Atom_t ext, Potassco::Value_t val) {
     if (update()) {
-        auto atm = out_->find(ext);
-        if (atm.second && atm.first->hasUid()) {
-            Id_t offset = numeric_cast<Id_t>(atm.first - atm.second->begin());
-            Output::External external(Output::LiteralId{NAF::POS, Output::AtomType::Predicate, offset, atm.second->domainOffset()}, val);
-            out_->output(external);
+        if (auto backend = out_->backend_()) {
+            backend->external(ext, val);
         }
     }
 }
 
+bool ClingoControl::isConflicting() noexcept {
+    return !clasp_->ok();
+}
+
 Potassco::AbstractStatistics *ClingoControl::statistics() {
     return clasp_->getStats();
 }
@@ -439,15 +481,13 @@ Potassco::AbstractStatistics *ClingoControl::statistics() {
 void ClingoControl::useEnumAssumption(bool enable) {
     enableEnumAssupmption_ = enable;
 }
+
 bool ClingoControl::useEnumAssumption() {
     return enableEnumAssupmption_;
 }
 
 SymbolicAtoms &ClingoControl::getDomain() {
-    if (clingoMode_) { return *this; }
-    else {
-        throw std::runtime_error("domain introspection only supported in clingo mode");
-    }
+    return *this;
 }
 
 namespace {
@@ -531,7 +571,7 @@ bool ClingoControl::fact(SymbolicAtomIter it) const {
 
 bool ClingoControl::external(SymbolicAtomIter it) const {
     auto &elem = domainElem(out_->predDoms(), it);
-    return elem.hasUid() && elem.isExternal() && static_cast<Clasp::Asp::LogicProgram*>(clasp_->program())->isExternal(elem.uid());
+    return elem.hasUid() && elem.isExternal() && (!clingoMode_ || static_cast<Clasp::Asp::LogicProgram*>(clasp_->program())->isExternal(elem.uid()));
 }
 
 SymbolicAtomIter ClingoControl::next(SymbolicAtomIter it) {
@@ -591,7 +631,20 @@ size_t ClingoControl::length() const {
     return ret;
 }
 
-Backend *ClingoControl::backend() { return out_->backend(); }
+bool ClingoControl::beginAddBackend() {
+    backend_ = out_->backend(logger());
+    return backend_ != nullptr;
+}
+
+Id_t ClingoControl::addAtom(Symbol sym) {
+    return out_->addAtom(sym);
+}
+
+void ClingoControl::endAddBackend() {
+    out_->endGround(logger());
+    backend_ = nullptr;
+}
+
 Potassco::Atom_t ClingoControl::addProgramAtom() { return out_->data.newAtom(); }
 
 ClingoControl::~ClingoControl() noexcept = default;
@@ -612,9 +665,9 @@ SolveResult convert(Clasp::ClaspFacade::Result res) {
 }
 
 ClingoSolveFuture::ClingoSolveFuture(ClingoControl &ctl, Clasp::SolveMode_t mode)
-: model_(ctl)
-, handle_(model_.context().clasp_->solve(mode)) {
-}
+: model_{ctl}
+, handle_{model_.context().clasp_->solve(mode)} { }
+
 SolveResult ClingoSolveFuture::get() {
     return convert(handle_.get());
 }
@@ -694,7 +747,8 @@ void ClingoLib::initOptions(Potassco::ProgramOptions::OptionContext& root) {
 }
 
 bool ClingoLib::onModel(Clasp::Solver const&, Clasp::Model const& m) {
-    return ClingoControl::onModel(m);
+    bool ret = ClingoControl::onModel(m);
+    return ret;
 }
 void ClingoLib::onEvent(Clasp::Event const& ev) {
     Clasp::ClaspFacade::StepReady const *r = Clasp::event_cast<Clasp::ClaspFacade::StepReady>(ev);
