@@ -29,6 +29,7 @@
 #pragma warning (disable : 4996) // 'strcpy' may be unsafe
 #endif
 
+#include <clingo/clingo_app.hh>
 #include <clingo/clingocontrol.hh>
 #include <clingo/script.h>
 #include <gringo/input/groundtermparser.hh>
@@ -74,6 +75,9 @@ void handleCXXError();
 #define GRINGO_CLINGO_TRY try
 #define GRINGO_CLINGO_CATCH catch (...) { handleCXXError(); return false; } return true
 
+#define GRINGO_CALLBACK_TRY try
+#define GRINGO_CALLBACK_CATCH(ref) catch (...){ (ref) = std::current_exception(); return false; } return true
+
 template <class F>
 size_t print_size(F f) {
     CountStream cs;
@@ -138,6 +142,13 @@ void handleCError(bool ret, std::exception_ptr *exc) {
     }
 }
 
+void forwardCError(bool ret, std::exception_ptr *exc) {
+    if (!ret) {
+        if (exc && *exc) { std::rethrow_exception(*exc); }
+        else             { throw ClingoError(); }
+    }
+}
+
 void handleCXXError() {
     try { throw; }
     catch (GringoError const &)         { g_lastException = std::current_exception(); g_lastCode = clingo_error_runtime; return; }
@@ -546,40 +557,6 @@ extern "C" bool clingo_theory_atoms_atom_to_string(clingo_theory_atoms_t *atoms,
     GRINGO_CLINGO_CATCH;
 }
 
-// {{{1 propagate init
-
-extern "C" bool clingo_propagate_init_solver_literal(clingo_propagate_init_t *init, clingo_literal_t lit, clingo_literal_t *ret) {
-    GRINGO_CLINGO_TRY { *ret = init->mapLit(lit); }
-    GRINGO_CLINGO_CATCH;
-}
-
-extern "C" bool clingo_propagate_init_add_watch(clingo_propagate_init_t *init, clingo_literal_t lit) {
-    GRINGO_CLINGO_TRY { init->addWatch(lit); }
-    GRINGO_CLINGO_CATCH;
-}
-
-extern "C" int clingo_propagate_init_number_of_threads(clingo_propagate_init_t *init) {
-    return init->threads();
-}
-
-extern "C" bool clingo_propagate_init_symbolic_atoms(clingo_propagate_init_t *init, clingo_symbolic_atoms_t **ret) {
-    GRINGO_CLINGO_TRY { *ret = &init->getDomain(); }
-    GRINGO_CLINGO_CATCH;
-}
-
-extern "C" bool clingo_propagate_init_theory_atoms(clingo_propagate_init_t *init, clingo_theory_atoms_t **ret) {
-    GRINGO_CLINGO_TRY { *ret = const_cast<clingo_theory_atoms*>(static_cast<clingo_theory_atoms const*>(&init->theory())); }
-    GRINGO_CLINGO_CATCH;
-}
-
-extern "C" void clingo_propagate_init_set_check_mode(clingo_propagate_init_t *init, clingo_propagator_check_mode_t mode) {
-    init->setCheckMode(mode);
-}
-
-extern "C" clingo_propagator_check_mode_t clingo_propagate_init_get_check_mode(clingo_propagate_init_t *init) {
-    return init->getCheckMode();
-}
-
 // {{{1 assignment
 
 struct clingo_assignment : public Potassco::AbstractAssignment { };
@@ -638,6 +615,49 @@ extern "C" bool clingo_assignment_is_total(clingo_assignment_t *assignment) {
     return assignment->isTotal();
 }
 
+// {{{1 propagate init
+
+extern "C" bool clingo_propagate_init_solver_literal(clingo_propagate_init_t *init, clingo_literal_t lit, clingo_literal_t *ret) {
+    GRINGO_CLINGO_TRY { *ret = init->mapLit(lit); }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_propagate_init_add_watch(clingo_propagate_init_t *init, clingo_literal_t lit) {
+    GRINGO_CLINGO_TRY { init->addWatch(lit); }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_propagate_init_add_watch_to_thread(clingo_propagate_init_t *init, clingo_literal_t lit, uint32_t thread_id) {
+    GRINGO_CLINGO_TRY { init->addWatch(thread_id, lit); }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" int clingo_propagate_init_number_of_threads(clingo_propagate_init_t *init) {
+    return init->threads();
+}
+
+extern "C" bool clingo_propagate_init_symbolic_atoms(clingo_propagate_init_t *init, clingo_symbolic_atoms_t **ret) {
+    GRINGO_CLINGO_TRY { *ret = &init->getDomain(); }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_propagate_init_theory_atoms(clingo_propagate_init_t *init, clingo_theory_atoms_t **ret) {
+    GRINGO_CLINGO_TRY { *ret = const_cast<clingo_theory_atoms*>(static_cast<clingo_theory_atoms const*>(&init->theory())); }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" void clingo_propagate_init_set_check_mode(clingo_propagate_init_t *init, clingo_propagator_check_mode_t mode) {
+    init->setCheckMode(mode);
+}
+
+extern "C" clingo_propagator_check_mode_t clingo_propagate_init_get_check_mode(clingo_propagate_init_t *init) {
+    return init->getCheckMode();
+}
+
+extern "C" clingo_assignment_t *clingo_propagate_init_assignment(clingo_propagate_init_t *init) {
+    return const_cast<clingo_assignment_t *>(static_cast<clingo_assignment_t const *>(&init->assignment()));
+}
+
 // {{{1 propagate control
 
 struct clingo_propagate_control : Potassco::AbstractSolver { };
@@ -647,7 +667,7 @@ extern "C" clingo_id_t clingo_propagate_control_thread_id(clingo_propagate_contr
 }
 
 extern "C" clingo_assignment_t *clingo_propagate_control_assignment(clingo_propagate_control_t *ctl) {
-    return const_cast<clingo_assignment *>(static_cast<clingo_assignment const *>(&ctl->assignment()));
+    return const_cast<clingo_assignment_t *>(static_cast<clingo_assignment_t const *>(&ctl->assignment()));
 }
 
 extern "C" bool clingo_propagate_control_add_clause(clingo_propagate_control_t *ctl, clingo_literal_t const *clause, size_t n, clingo_clause_type_t prop, bool *ret) {
@@ -682,7 +702,7 @@ extern "C" void clingo_propagate_control_remove_watch(clingo_propagate_control_t
 
 struct clingo_solve_control : clingo_model { };
 
-extern "C" bool clingo_model_thread_id(clingo_model_t *ctl, clingo_id_t *ret) {
+extern "C" bool clingo_model_thread_id(clingo_model_t const *ctl, clingo_id_t *ret) {
     GRINGO_CLINGO_TRY { *ret = ctl->threadId(); }
     GRINGO_CLINGO_CATCH;
 }
@@ -696,12 +716,12 @@ extern "C" bool clingo_solve_control_symbolic_atoms(clingo_solve_control_t *cont
     GRINGO_CLINGO_TRY { *atoms = &control->getDomain(); }
     GRINGO_CLINGO_CATCH;
 }
-extern "C" bool clingo_model_contains(clingo_model_t *m, clingo_symbol_t atom, bool *ret) {
+extern "C" bool clingo_model_contains(clingo_model_t const *m, clingo_symbol_t atom, bool *ret) {
     GRINGO_CLINGO_TRY { *ret = m->contains(Symbol(atom)); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_model_symbols_size(clingo_model_t *m, clingo_show_type_bitset_t show, size_t *n) {
+extern "C" bool clingo_model_symbols_size(clingo_model_t const *m, clingo_show_type_bitset_t show, size_t *n) {
     GRINGO_CLINGO_TRY {
         // TODO: implement matching C++ functions ...
         SymSpan atoms = m->atoms(show);
@@ -710,7 +730,7 @@ extern "C" bool clingo_model_symbols_size(clingo_model_t *m, clingo_show_type_bi
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_model_symbols(clingo_model_t *m, clingo_show_type_bitset_t show, clingo_symbol_t *ret, size_t n) {
+extern "C" bool clingo_model_symbols(clingo_model_t const *m, clingo_show_type_bitset_t show, clingo_symbol_t *ret, size_t n) {
     GRINGO_CLINGO_TRY {
         // TODO: implement matching C++ functions ...
         SymSpan atoms = m->atoms(show);
@@ -720,12 +740,12 @@ extern "C" bool clingo_model_symbols(clingo_model_t *m, clingo_show_type_bitset_
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_model_optimality_proven(clingo_model_t *m, bool *proven) {
+extern "C" bool clingo_model_optimality_proven(clingo_model_t const *m, bool *proven) {
     GRINGO_CLINGO_TRY { *proven = m->optimality_proven(); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_model_cost_size(clingo_model_t *m, size_t *n) {
+extern "C" bool clingo_model_cost_size(clingo_model_t const *m, size_t *n) {
     GRINGO_CLINGO_TRY {
         // TODO: implement matching C++ functions ...
         auto opt = m->optimization();
@@ -734,7 +754,7 @@ extern "C" bool clingo_model_cost_size(clingo_model_t *m, size_t *n) {
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_model_cost(clingo_model_t *m, int64_t *ret, size_t n) {
+extern "C" bool clingo_model_cost(clingo_model_t const *m, int64_t *ret, size_t n) {
     GRINGO_CLINGO_TRY {
         // TODO: implement matching C++ functions ...
         auto opt = m->optimization();
@@ -744,22 +764,27 @@ extern "C" bool clingo_model_cost(clingo_model_t *m, int64_t *ret, size_t n) {
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_model_context(clingo_model_t *m, clingo_solve_control_t **ret) {
-    GRINGO_CLINGO_TRY { *ret = static_cast<clingo_solve_control_t*>(m); }
+extern "C" bool clingo_model_extend(clingo_model_t *model, clingo_symbol_t const *symbols, size_t size) {
+    GRINGO_CLINGO_TRY { model->add({reinterpret_cast<Symbol const *>(symbols), size}); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_model_number(clingo_model_t *m, uint64_t *n) {
+extern "C" bool clingo_model_context(clingo_model_t const *m, clingo_solve_control_t **ret) {
+    GRINGO_CLINGO_TRY { *ret = static_cast<clingo_solve_control_t*>(const_cast<clingo_model_t *>(m)); }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_model_number(clingo_model_t const *m, uint64_t *n) {
     GRINGO_CLINGO_TRY { *n = m->number(); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_model_type(clingo_model_t *m, clingo_model_type_t *ret) {
+extern "C" bool clingo_model_type(clingo_model_t const *m, clingo_model_type_t *ret) {
     GRINGO_CLINGO_TRY { *ret = static_cast<clingo_model_type_t>(m->type()); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_model_is_true(clingo_model_t *m, clingo_literal_t lit, bool *result) {
+extern "C" bool clingo_model_is_true(clingo_model_t const *m, clingo_literal_t lit, bool *result) {
     GRINGO_CLINGO_TRY { *result = static_cast<clingo_model_type_t>(m->isTrue(lit)); }
     GRINGO_CLINGO_CATCH;
 }
@@ -871,46 +896,67 @@ extern "C" bool clingo_configuration_value_is_assigned(clingo_configuration_t *c
 
 struct clingo_statistic : public Potassco::AbstractStatistics { };
 
-extern "C" bool clingo_statistics_root(clingo_statistics_t *stats, uint64_t *ret) {
+extern "C" bool clingo_statistics_root(const clingo_statistics_t *stats, uint64_t *ret) {
     GRINGO_CLINGO_TRY { *ret = stats->root(); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_statistics_type(clingo_statistics_t *stats, uint64_t key, clingo_statistics_type_t *ret) {
+extern "C" bool clingo_statistics_type(const clingo_statistics_t *stats, uint64_t key, clingo_statistics_type_t *ret) {
     GRINGO_CLINGO_TRY { *ret = stats->type(key); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_statistics_array_size(clingo_statistics_t *stats, uint64_t key, size_t *ret) {
+extern "C" bool clingo_statistics_array_size(const clingo_statistics_t *stats, uint64_t key, size_t *ret) {
     GRINGO_CLINGO_TRY { *ret = stats->size(key); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_statistics_array_at(clingo_statistics_t *stats, uint64_t key, size_t index, uint64_t *ret) {
+extern "C" bool clingo_statistics_array_at(const clingo_statistics_t *stats, uint64_t key, size_t index, uint64_t *ret) {
     GRINGO_CLINGO_TRY { *ret = stats->at(key, index); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_statistics_map_size(clingo_statistics_t *stats, uint64_t key, size_t *n) {
+extern "C" bool clingo_statistics_array_push(clingo_statistics_t *stats, uint64_t key, clingo_statistics_type_t type, uint64_t *ret) {
+    GRINGO_CLINGO_TRY { *ret = stats->push(key, static_cast<Potassco::Statistics_t>(type)); }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_statistics_map_size(const clingo_statistics_t *stats, uint64_t key, size_t *n) {
     GRINGO_CLINGO_TRY { *n = stats->size(key); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_statistics_map_subkey_name(clingo_statistics_t *stats, uint64_t key, size_t index, char const **name) {
+extern "C" bool clingo_statistics_map_has_subkey(const clingo_statistics_t *stats, uint64_t key, char const *name, bool* result) {
+    GRINGO_CLINGO_TRY { uint64_t temp; *result = stats->find(key, name, &temp); }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_statistics_map_subkey_name(const clingo_statistics_t *stats, uint64_t key, size_t index, char const **name) {
     GRINGO_CLINGO_TRY { *name = stats->key(key, index); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_statistics_map_at(clingo_statistics_t *stats, uint64_t key, char const *name, uint64_t *ret) {
+extern "C" bool clingo_statistics_map_at(const clingo_statistics_t *stats, uint64_t key, char const *name, uint64_t *ret) {
     GRINGO_CLINGO_TRY { *ret = stats->get(key, name); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_statistics_value_get(clingo_statistics_t *stats, uint64_t key, double *value) {
+extern "C" bool clingo_statistics_map_add_subkey(clingo_statistics_t *stats, uint64_t key, char const *name, clingo_statistics_type_t type, uint64_t *ret) {
+    GRINGO_CLINGO_TRY { *ret = stats->add(key, name, static_cast<Potassco::Statistics_t>(type)); }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_statistics_value_get(const clingo_statistics_t *stats, uint64_t key, double *value) {
     GRINGO_CLINGO_TRY { *value = stats->value(key); }
     GRINGO_CLINGO_CATCH;
 }
 
+extern "C" bool clingo_statistics_value_set(clingo_statistics_t *stats, uint64_t key, double value) {
+    GRINGO_CLINGO_TRY { stats->set(key, value); }
+    GRINGO_CLINGO_CATCH;
+}
+
+
 // {{{1 global functions
 
 extern "C" bool clingo_parse_term(char const *str, clingo_logger_t logger, void *data, unsigned message_limit, clingo_symbol_t *ret) {
@@ -958,48 +1004,60 @@ extern "C" void clingo_version(int *major, int *minor, int *revision) {
 
 struct clingo_backend : clingo_control_t { };
 
+extern "C" bool clingo_backend_begin(clingo_backend_t *backend) {
+    GRINGO_CLINGO_TRY {
+        if (!backend->beginAddBackend()) { throw std::runtime_error("backend not available"); }
+    }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" bool clingo_backend_end(clingo_backend_t *backend) {
+    GRINGO_CLINGO_TRY { backend->endAddBackend(); }
+    GRINGO_CLINGO_CATCH;
+}
+
 extern "C" bool clingo_backend_rule(clingo_backend_t *backend, bool choice, clingo_atom_t const *head, size_t head_n, clingo_literal_t const *body, size_t body_n) {
-    GRINGO_CLINGO_TRY { outputRule(*backend->backend(), choice, {head, head_n}, {body, body_n}); }
+    GRINGO_CLINGO_TRY { outputRule(*backend->getBackend(), choice, {head, head_n}, {body, body_n}); }
     GRINGO_CLINGO_CATCH;
 }
 
 extern "C" bool clingo_backend_weight_rule(clingo_backend_t *backend, bool choice, clingo_atom_t const *head, size_t head_n, clingo_weight_t lower, clingo_weighted_literal_t const *body, size_t body_n) {
-    GRINGO_CLINGO_TRY { outputRule(*backend->backend(), choice, {head, head_n}, lower, {reinterpret_cast<Potassco::WeightLit_t const *>(body), body_n}); }
+    GRINGO_CLINGO_TRY { outputRule(*backend->getBackend(), choice, {head, head_n}, lower, {reinterpret_cast<Potassco::WeightLit_t const *>(body), body_n}); }
     GRINGO_CLINGO_CATCH;
 }
 
 extern "C" bool clingo_backend_minimize(clingo_backend_t *backend, clingo_weight_t prio, clingo_weighted_literal_t const* lits, size_t lits_n) {
-    GRINGO_CLINGO_TRY { backend->backend()->minimize(prio, {reinterpret_cast<Potassco::WeightLit_t const *>(lits), lits_n}); }
+    GRINGO_CLINGO_TRY { backend->getBackend()->minimize(prio, {reinterpret_cast<Potassco::WeightLit_t const *>(lits), lits_n}); }
     GRINGO_CLINGO_CATCH;
 }
 
 extern "C" bool clingo_backend_project(clingo_backend_t *backend, clingo_atom_t const *atoms, size_t n) {
-    GRINGO_CLINGO_TRY { backend->backend()->project({atoms, n}); }
+    GRINGO_CLINGO_TRY { backend->getBackend()->project({atoms, n}); }
     GRINGO_CLINGO_CATCH;
 }
 
 extern "C" bool clingo_backend_external(clingo_backend_t *backend, clingo_atom_t atom, clingo_external_type_t v) {
-    GRINGO_CLINGO_TRY { backend->backend()->external(atom, Potassco::Value_t(v)); }
+    GRINGO_CLINGO_TRY { backend->getBackend()->external(atom, Potassco::Value_t(v)); }
     GRINGO_CLINGO_CATCH;
 }
 
 extern "C" bool clingo_backend_assume(clingo_backend_t *backend, clingo_literal_t const *literals, size_t n) {
-    GRINGO_CLINGO_TRY { backend->backend()->assume({literals, n}); }
+    GRINGO_CLINGO_TRY { backend->getBackend()->assume({literals, n}); }
     GRINGO_CLINGO_CATCH;
 }
 
 extern "C" bool clingo_backend_heuristic(clingo_backend_t *backend, clingo_atom_t atom, clingo_heuristic_type_t type, int bias, unsigned priority, clingo_literal_t const *condition, size_t condition_n) {
-    GRINGO_CLINGO_TRY { backend->backend()->heuristic(atom, Potassco::Heuristic_t(type), bias, priority, {condition, condition_n}); }
+    GRINGO_CLINGO_TRY { backend->getBackend()->heuristic(atom, Potassco::Heuristic_t(type), bias, priority, {condition, condition_n}); }
     GRINGO_CLINGO_CATCH;
 }
 
 extern "C" bool clingo_backend_acyc_edge(clingo_backend_t *backend, int node_u, int node_v, clingo_literal_t const *condition, size_t condition_n) {
-    GRINGO_CLINGO_TRY { backend->backend()->acycEdge(node_u, node_v, {condition, condition_n}); }
+    GRINGO_CLINGO_TRY { backend->getBackend()->acycEdge(node_u, node_v, {condition, condition_n}); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_backend_add_atom(clingo_backend_t *backend, clingo_atom_t *ret) {
-    GRINGO_CLINGO_TRY { *ret = backend->addProgramAtom(); }
+extern "C" bool clingo_backend_add_atom(clingo_backend_t *backend, clingo_symbol_t *symbol, clingo_atom_t *ret) {
+    GRINGO_CLINGO_TRY { *ret = symbol ? backend->addAtom(Symbol{*symbol}) : backend->addProgramAtom(); }
     GRINGO_CLINGO_CATCH;
 }
 
@@ -1024,7 +1082,7 @@ extern "C" bool clingo_solve_handle_close(clingo_solve_handle_t *handle) {
     GRINGO_CLINGO_TRY { if (handle) { delete handle; } }
     GRINGO_CLINGO_CATCH;
 }
-extern "C" bool clingo_solve_handle_model(clingo_solve_handle_t *handle, clingo_model_t **model) {
+extern "C" bool clingo_solve_handle_model(clingo_solve_handle_t *handle, clingo_model_t const **model) {
     GRINGO_CLINGO_TRY {
         *model = const_cast<Model*>(handle->model());
     }
@@ -1126,14 +1184,6 @@ extern "C" bool clingo_control_ground(clingo_control_t *ctl, clingo_part_t const
 
 namespace {
 
-Control::Assumptions toAss(clingo_symbolic_literal_t const * assumptions, size_t n) {
-    Control::Assumptions ass;
-    for (auto it = assumptions, ie = it + n; it != ie; ++it) {
-        ass.emplace_back(static_cast<Symbol const>(it->symbol), it->positive);
-    }
-    return ass;
-}
-
 class ClingoSolveEventHandler : public SolveEventHandler {
 public:
     ClingoSolveEventHandler(clingo_solve_event_callback_t cb, void *data)
@@ -1145,8 +1195,12 @@ private:
         if (!cb_(clingo_solve_event_type_model, &model, data_, &goon)) { throw ClingoError(); }
         return goon;
     }
-    void on_finish(SolveResult ret) override {
+    void on_finish(SolveResult ret, Potassco::AbstractStatistics *step, Potassco::AbstractStatistics *accu) override {
         bool goon = true;
+        clingo_statistics_t *stats[] = {static_cast<clingo_statistics_t*>(step), static_cast<clingo_statistics_t*>(accu)};
+        if (!cb_(clingo_solve_event_type_statistics, &stats, data_, &goon)) {
+            clingo_terminate("error in SolveEventHandler::on_statistics going to terminate");
+        }
         if (!cb_(clingo_solve_event_type_finish, &ret, data_, &goon)) {
             clingo_terminate("error in SolveEventHandler::on_finish going to terminate");
         }
@@ -1158,22 +1212,29 @@ private:
 
 } // namespace
 
-extern "C" bool clingo_control_solve(clingo_control_t *control, clingo_solve_mode_bitset_t mode, clingo_symbolic_literal_t const *assumptions, size_t assumptions_size, clingo_solve_event_callback_t notify, void *data, clingo_solve_handle_t **handle) {
+extern "C" bool clingo_control_solve(clingo_control_t *control, clingo_solve_mode_bitset_t mode, clingo_literal_t const *assumptions, size_t assumptions_size, clingo_solve_event_callback_t notify, void *data, clingo_solve_handle_t **handle) {
     GRINGO_CLINGO_TRY { *handle = static_cast<clingo_solve_handle_t*>(control->solve(
-        toAss(assumptions, assumptions_size),
+        Potassco::toSpan(assumptions, assumptions_size),
         mode,
         notify ? gringo_make_unique<ClingoSolveEventHandler>(notify, data) : nullptr
     ).release()); }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_control_assign_external(clingo_control_t *ctl, clingo_symbol_t atom, clingo_truth_value_t value) {
-    GRINGO_CLINGO_TRY { ctl->assignExternal(Symbol(atom), static_cast<Potassco::Value_t>(value)); }
+extern "C" bool clingo_control_assign_external(clingo_control_t *ctl, clingo_literal_t literal, clingo_truth_value_t value) {
+    GRINGO_CLINGO_TRY {
+        if (literal < 0) {
+            literal = -literal;
+            if      (value == Potassco::Value_t::True)  { value = Potassco::Value_t::False; }
+            else if (value == Potassco::Value_t::False) { value = Potassco::Value_t::True; }
+        }
+        ctl->assignExternal(literal, static_cast<Potassco::Value_t>(value));
+    }
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_control_release_external(clingo_control_t *ctl, clingo_symbol_t atom) {
-    GRINGO_CLINGO_TRY { ctl->assignExternal(Symbol(atom), Potassco::Value_t::Release); }
+extern "C" bool clingo_control_release_external(clingo_control_t *ctl, clingo_literal_t literal) {
+    GRINGO_CLINGO_TRY { ctl->assignExternal(std::abs(literal), Potassco::Value_t::Release); }
     GRINGO_CLINGO_CATCH;
 }
 
@@ -1262,10 +1323,7 @@ extern "C" bool clingo_control_use_enumeration_assumption(clingo_control_t *ctl,
 }
 
 extern "C" bool clingo_control_backend(clingo_control_t *ctl, clingo_backend_t **ret) {
-    GRINGO_CLINGO_TRY {
-        if (ctl->backend()) { *ret = static_cast<clingo_backend_t*>(ctl); }
-        else { throw std::runtime_error("backend not available"); }
-    }
+    GRINGO_CLINGO_TRY { *ret = static_cast<clingo_backend_t*>(ctl); }
     GRINGO_CLINGO_CATCH;
 }
 
@@ -1274,7 +1332,11 @@ extern "C" bool clingo_control_configuration(clingo_control_t *ctl, clingo_confi
     GRINGO_CLINGO_CATCH;
 }
 
-extern "C" bool clingo_control_statistics(clingo_control_t *ctl, clingo_statistics_t **stats) {
+extern "C" bool clingo_control_is_conflicting(clingo_control_t *control) {
+    return control->isConflicting();
+}
+
+extern "C" bool clingo_control_statistics(clingo_control_t *ctl, const clingo_statistics_t **stats) {
     GRINGO_CLINGO_TRY { *stats = static_cast<clingo_statistics_t*>(ctl->statistics()); }
     GRINGO_CLINGO_CATCH;
 }
@@ -1402,19 +1464,16 @@ private:
         using Data = std::pair<SymVec, std::exception_ptr>;
         Data data;
         auto l = conv(loc);
-        handleCError(script_.call(
+        forwardCError(script_.call(
             &l, name.c_str(), reinterpret_cast<clingo_symbol_t const *>(args.first), args.size,
-            [](clingo_symbol_t const *symbols, size_t symbols_size, void *data) {
-                try {
+            [](clingo_symbol_t const *symbols, size_t symbols_size, void *pdata) {
+                auto &data = *static_cast<Data*>(pdata);
+                GRINGO_CALLBACK_TRY {
                     for (auto it = symbols, ie = it + symbols_size; it != ie; ++it) {
-                        static_cast<SymVec*>(data)->emplace_back(Symbol{*it});
+                        data.first.emplace_back(Symbol{*it});
                     }
-                    return true;
-                }
-                catch (...) {
-                    handleCXXError();
-                    return false;
                 }
+                GRINGO_CALLBACK_CATCH(data.second);
             },
             &data, data_), &data.second);
         return data.first;
@@ -1446,6 +1505,120 @@ extern "C" CLINGO_VISIBILITY_DEFAULT char const *clingo_script_version_(clingo_a
     return g_scripts().version(static_cast<clingo_ast_script_type>(type));
 }
 
+extern "C" CLINGO_VISIBILITY_DEFAULT int clingo_main_(int argc, char *argv[]) {
+    Gringo::ClingoApp app;
+    return app.main(argc, argv);
+}
+
+char *str_duplicate(char const *str) {
+    char *ret = new char[strlen(str) + 1];
+    std::strcpy(ret, str);
+    return ret;
+}
+
+struct clingo_options : ClingoApp {};
+
+namespace {
+
+class CClingoApp : public IClingoApp {
+public:
+    CClingoApp(clingo_application_t app, void *data)
+    : app_(app)
+    , data_{data} {
+        name_ = app_.program_name ? app_.program_name(data_) : IClingoApp::program_name();
+        version_ = app_.version ? app_.version(data_) : IClingoApp::version();
+    }
+    unsigned message_limit() const override {
+        if (app_.message_limit) {
+            return app_.message_limit(data_);
+        }
+        else {
+            return IClingoApp::message_limit();
+        }
+    }
+    char const *program_name() const override {
+        return name_;
+    }
+    char const *version() const override {
+        return version_;
+    }
+    bool has_main() const override {
+        return app_.main;
+    }
+    void main(ClingoControl &ctl, std::vector<std::string> const &files) override {
+        assert(has_main());
+        std::vector<char const *> c_files;
+        for (auto &x : files) {
+            c_files.emplace_back(x.c_str());
+        }
+        handleCError(app_.main(&ctl, c_files.data(), c_files.size(), data_));
+    }
+    bool has_log() const override { return app_.logger; }
+    void log(Gringo::Warnings code, char const *message) noexcept override {
+        assert(has_log());
+        app_.logger(static_cast<clingo_warning_t>(code), message, data_);
+    }
+    bool has_printer() const override { return app_.printer; }
+    void print_model(Model *model, std::function<void()> printer) override {
+        handleCError(app_.printer(model, [](void *data) {
+            GRINGO_CLINGO_TRY {
+                (*static_cast<std::function<void()>*>(data))();
+            }
+            GRINGO_CLINGO_CATCH;
+        }, &printer, data_));
+    }
+
+    void register_options(ClingoApp &app) override {
+        if (app_.register_options) {
+            handleCError(app_.register_options(static_cast<clingo_options_t*>(&app), data_));
+        }
+    }
+    void validate_options() override {
+        if (app_.validate_options) {
+            handleCError(app_.validate_options(data_));
+        }
+    }
+private:
+    clingo_application_t app_;
+    void *data_;
+    char const *name_;
+    char const *version_;
+};
+
+} // namespace
+
+extern "C" CLINGO_VISIBILITY_DEFAULT bool clingo_options_add(clingo_options_t *options, char const *group, char const *option, char const *description, bool (*parse) (char const *value, void *data), void *data, bool multi, char const *argument) {
+    GRINGO_CLINGO_TRY {
+        options->addOption(group, option, description, [parse, data](char const *value) { return parse(value, data); }, argument, multi);
+    }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" CLINGO_VISIBILITY_DEFAULT bool clingo_options_add_flag(clingo_options_t *options, char const *group, char const *option, char const *description, bool *target) {
+    GRINGO_CLINGO_TRY { options->addFlag(group, option, description, *target); }
+    GRINGO_CLINGO_CATCH;
+}
+
+extern "C" CLINGO_VISIBILITY_DEFAULT int clingo_main(clingo_application *application, char const *const * arguments, size_t size, void *data) {
+    try {
+        UIClingoApp app = std::make_unique<CClingoApp>(*application, data);
+        std::vector<std::unique_ptr<char[]>> args_buf;
+        std::vector<char *> args;
+        args_buf.emplace_back(str_duplicate(app->program_name()));
+        for (auto arg = arguments, end = arguments + size; arg != end; ++arg) {
+            args_buf.emplace_back(str_duplicate(*arg));
+        }
+        args_buf.emplace_back(nullptr);
+        for (auto &x : args_buf) { args.emplace_back(x.get()); }
+        return Gringo::ClingoApp{std::move(app)}.main(args.size() - 1, args.data());
+    }
+    catch (...) {
+        handleCXXError();
+        std::cerr << "error during initialization: going to terminate:\n" << clingo_error_message() << std::endl;
+        std::terminate();
+    }
+}
+
 // }}}1
 
 #if defined(__GNUC__) && !defined(__clang__)
