@@ -153,11 +153,15 @@ struct IncrementalControl : Control {
         }
         if (!parts.empty()) {
             Ground::Parameters params;
-            for (auto &x : parts) { params.add(x.first, SymVec(x.second)); }
-            Ground::Program gPrg(prg.toGround(out.data, logger_));
+            std::set<Sig> sigs;
+            for (auto &x : parts) {
+                params.add(x.first, SymVec(x.second));
+                sigs.emplace(x.first, numeric_cast<uint32_t>(x.second.size()), false);
+            }
+            Ground::Program gPrg(prg.toGround(sigs, out.data, logger_));
             LOG << "************* intermediate program *************" << std::endl << gPrg << std::endl;
             LOG << "*************** grounded program ***************" << std::endl;
-            gPrg.ground(params, scripts, out, false, logger_);
+            gPrg.ground(params, scripts, out, logger_);
         }
     }
     void add(std::string const &name, StringVec const &params, std::string const &part) override {
@@ -182,10 +186,9 @@ struct IncrementalControl : Control {
         parse();
     }
     bool blocked() override { return false; }
-    USolveFuture solve(Assumptions &&ass, clingo_solve_mode_bitset_t, USolveEventHandler cb) override {
-        out.assume(std::move(ass));
+    USolveFuture solve(Assumptions ass, clingo_solve_mode_bitset_t, USolveEventHandler cb) override {
         grounded = false;
-        out.endStep(true, logger_);
+        out.endStep(ass);
         out.reset(true);
         return gringo_make_unique<DefaultSolveFuture>(std::move(cb));
     }
@@ -206,13 +209,9 @@ struct IncrementalControl : Control {
         out.registerObserver(std::move(prg), replace);
     }
     Potassco::AbstractStatistics *statistics() override { throw std::runtime_error("statistics not supported (yet)"); }
-    void assignExternal(Symbol ext, Potassco::Value_t val) override {
-        auto atm = out.find(ext);
-        if (atm.second && atm.first->hasUid()) {
-            Id_t offset = numeric_cast<Id_t>(atm.first - atm.second->begin());
-            Output::External external(Output::LiteralId{NAF::POS, Output::AtomType::Predicate, offset, atm.second->domainOffset()}, val);
-            out.output(external);
-        }
+    bool isConflicting() noexcept override { return false; }
+    void assignExternal(Potassco::Atom_t ext, Potassco::Value_t val) override {
+        if (auto *b = out.backend_()) { b->external(ext, val); }
     }
     SymbolicAtoms &getDomain() override { throw std::runtime_error("domain introspection not supported"); }
     ConfigProxy &getConf() override { throw std::runtime_error("configuration not supported"); }
@@ -222,7 +221,19 @@ struct IncrementalControl : Control {
     virtual ~IncrementalControl() { }
     Output::DomainData const &theory() const override { return out.data; }
     void cleanupDomains() override { }
-    Backend *backend() override { return out.backend(); }
+    bool beginAddBackend() override {
+        backend_ = out.backend(logger());
+        return backend_ != nullptr;
+    }
+    Backend *getBackend() override {
+        if (!backend_) { throw std::runtime_error("backend not available"); }
+        return backend_;
+    }
+    Id_t addAtom(Symbol sym) override { return out.addAtom(sym); }
+    void endAddBackend() override {
+        out.endGround(logger());
+        backend_ = nullptr;
+    }
     Potassco::Atom_t addProgramAtom() override { return out.data.newAtom(); }
     Input::GroundTermParser        termParser;
     Output::OutputBase            &out;
@@ -233,6 +244,7 @@ struct IncrementalControl : Control {
     Input::NonGroundParser         parser;
     GringoOptions const           &opts;
     Logger                         logger_;
+    Backend                       *backend_ = nullptr;
     std::unique_ptr<Input::NongroundProgramBuilder> builder;
     bool                                   incmode = false;
     bool                                   parsed = false;
@@ -282,7 +294,7 @@ static bool parseText(const std::string&, GringoOptions& out) {
 struct GringoApp : public Potassco::Application {
     using StringSeq = std::vector<std::string>;
     virtual const char* getName() const    { return "gringo"; }
-    virtual const char* getVersion() const { return CLINGO_VERSION; }
+    virtual const char* getVersion() const { return CLINGO_VERSION_STRING; }
     virtual HelpOpt     getHelpOption() const { return HelpOpt("Print (<n {1=default|2=advanced}) help and exit", 2); }
     virtual void initOptions(Potassco::ProgramOptions::OptionContext& root) {
         using namespace Potassco::ProgramOptions;
@@ -346,7 +358,7 @@ struct GringoApp : public Potassco::Application {
         printUsage();
         Potassco::ProgramOptions::FileOut out(stdout);
         root.description(out);
-		printf("\nType '%s --help=2' for further options.\n", getName());
+        printf("\nType '%s --help=2' for further options.\n", getName());
         printf("\n");
         printUsage();
     }
@@ -380,7 +392,7 @@ struct GringoApp : public Potassco::Application {
             parts.emplace_back("base", SymVec{});
             inc.incremental_ = false;
             inc.ground(parts, nullptr);
-            inc.solve({}, 0, nullptr)->get();
+            inc.solve({nullptr, 0}, 0, nullptr)->get();
         }
     }
 
