@@ -107,7 +107,7 @@ TEST_CASE("solving", "[clingo]") {
                 REQUIRE(messages.empty());
             }
         }
-        SECTION("statistics") {
+        SECTION("get_statistics") {
             ctl.add("pigeon", {"p", "h"}, "1 {p(P,H) : P=1..p}1 :- H=1..h."
                                           "1 {p(P,H) : H=1..h}1 :- P=1..p.");
             ctl.ground({{"pigeon", {Number(6), Number(5)}}});
@@ -134,6 +134,31 @@ TEST_CASE("solving", "[clingo]") {
             }
             REQUIRE(nloop == 1);
         }
+        SECTION("set_statistics") {
+            ctl.ground({{"base", {}}});
+            struct H : Clingo::SolveEventHandler {
+                void on_statistics(Clingo::UserStatistics step, Clingo::UserStatistics accu) override {
+                    auto map = step.add_subkey("map", StatisticsType::Map);
+                    map.add_subkey("x", StatisticsType::Value).set_value(1);
+                    map.add_subkey("y", StatisticsType::Value).set_value(2);
+                    map = accu.add_subkey("map", StatisticsType::Map);
+                    map.add_subkey("x", StatisticsType::Value).set_value(3);
+                    map.add_subkey("y", StatisticsType::Value).set_value(4);
+                }
+            } handler;
+            REQUIRE(test_solve(ctl.solve(Clingo::LiteralSpan{}, &handler), models).is_satisfiable());
+            auto stats = ctl.statistics();
+            REQUIRE(stats["user_step.map"].type() == StatisticsType::Map);
+            REQUIRE(stats["user_step.map.x"].type() == StatisticsType::Value);
+            REQUIRE(stats["user_step.map.y"].type() == StatisticsType::Value);
+            REQUIRE(stats["user_step.map.x"].value() == 1);
+            REQUIRE(stats["user_step.map.y"].value() == 2);
+            REQUIRE(stats["user_accu.map"].type() == StatisticsType::Map);
+            REQUIRE(stats["user_accu.map.x"].type() == StatisticsType::Value);
+            REQUIRE(stats["user_accu.map.y"].type() == StatisticsType::Value);
+            REQUIRE(stats["user_accu.map.x"].value() == 3);
+            REQUIRE(stats["user_accu.map.y"].value() == 4);
+        }
         SECTION("configuration") {
             auto conf = ctl.configuration();
             std::vector<std::string> keys_root;
@@ -163,16 +188,26 @@ TEST_CASE("solving", "[clingo]") {
         }
         SECTION("backend") {
             // NOTE: ground has to be called before using the backend
-            auto backend = ctl.backend();
-            literal_t a = backend.add_atom(), b = backend.add_atom();
-            backend.rule(true, {atom_t(a)}, {});
-            backend.rule(false, {atom_t(b)}, {-a});
+            literal_t a, b;
+            {
+                auto backend = ctl.backend();
+                a = backend.add_atom();
+                b = backend.add_atom();
+                backend.rule(true, {atom_t(a)}, {});
+                backend.rule(false, {atom_t(b)}, {-a});
+            }
             test_solve(ctl.solve(), models);
             REQUIRE(models == (ModelVec{{},{}}));
-            backend.assume({a});
+            {
+                auto backend = ctl.backend();
+                backend.assume({a});
+            }
             test_solve(ctl.solve(), models);
             REQUIRE(models == (ModelVec{{}}));
-            backend.minimize(1, {{a,1},{b,1}});
+            {
+                auto backend = ctl.backend();
+                backend.minimize(1, {{a,1},{b,1}});
+            }
             test_solve(ctl.solve(), models);
             REQUIRE(ctl.statistics()["summary.costs"].size() == 1);
             REQUIRE(ctl.statistics()["summary.costs"][size_t(0)] == 1);
@@ -182,40 +217,77 @@ TEST_CASE("solving", "[clingo]") {
         }
         SECTION("backend-project") {
             ctl.configuration()["solve.project"] = "auto";
-            auto backend = ctl.backend();
-            atom_t a = backend.add_atom(), b = backend.add_atom();
-            backend.rule(true, {a, b}, {});
-            backend.project({a});
+            {
+                auto backend = ctl.backend();
+                atom_t a = backend.add_atom(), b = backend.add_atom();
+                backend.rule(true, {a, b}, {});
+                backend.project({a});
+            }
             test_solve(ctl.solve(), models);
             REQUIRE(models == (ModelVec{{},{}}));
         }
         SECTION("backend-external") {
-            auto backend = ctl.backend();
-            atom_t a = backend.add_atom();
-            backend.external(a, ExternalType::Free);
-            test_solve(ctl.solve(), models);
-            REQUIRE(models == (ModelVec{{},{}}));
-            backend.external(a, ExternalType::Release);
+            {
+                auto backend = ctl.backend();
+                atom_t a = backend.add_atom();
+                backend.external(a, ExternalType::Free);
+                test_solve(ctl.solve(), models);
+                REQUIRE(models == (ModelVec{{},{}}));
+                backend.external(a, ExternalType::Release);
+            }
             test_solve(ctl.solve(), models);
             REQUIRE(models == (ModelVec{{}}));
         }
         SECTION("backend-acyc") {
-            auto backend = ctl.backend();
-            atom_t a = backend.add_atom(), b = backend.add_atom();
-            backend.rule(true, {a, b}, {});
-            backend.acyc_edge(1, 2, {literal_t(a)});
-            backend.acyc_edge(2, 1, {literal_t(b)});
+            {
+                auto backend = ctl.backend();
+                atom_t a = backend.add_atom(), b = backend.add_atom();
+                backend.rule(true, {a, b}, {});
+                backend.acyc_edge(1, 2, {literal_t(a)});
+                backend.acyc_edge(2, 1, {literal_t(b)});
+            }
             test_solve(ctl.solve(), models);
             REQUIRE(models == (ModelVec{{},{},{}}));
         }
         SECTION("backend-weight-rule") {
-            auto backend = ctl.backend();
-            atom_t a = backend.add_atom(), b = backend.add_atom();
-            backend.rule(true, {a, b}, {});
-            backend.weight_rule(false, {}, 1, {{literal_t(a),1}, {literal_t(b),1}});
+            {
+                auto backend = ctl.backend();
+                atom_t a = backend.add_atom(), b = backend.add_atom();
+                backend.rule(true, {a, b}, {});
+                backend.weight_rule(false, {}, 1, {{literal_t(a),1}, {literal_t(b),1}});
+            }
             test_solve(ctl.solve(), models);
             REQUIRE(models == (ModelVec{{}}));
         }
+        SECTION("backend-add-atom") {
+            auto a = [](int x) { return Function("a", {Number(x)}); };
+            auto b = [](int x) { return Function("b", {Number(x)}); };
+            auto c = [](int x) { return Function("c", {Number(x)}); };
+            auto d = [](int x) { return Function("d", {Number(x)}); };
+            auto e = [](int x) { return Function("e", {Number(x)}); };
+            {
+                auto backend = ctl.backend();
+                auto aa = backend.add_atom(a(2));
+                auto ab = backend.add_atom(b(1));
+                backend.rule(true, {aa}, {});
+                backend.rule(true, {ab}, {});
+            }
+
+            ctl.ground({{"base", {}}});
+
+            {
+                auto backend = ctl.backend();
+                auto ae = backend.add_atom(e(3));
+                backend.rule(false, {ae}, {});
+            }
+            test_solve(ctl.solve(), models);
+            REQUIRE(models == (ModelVec{{ a(2), b(1), e(3) }, { a(2), e(3) }, { b(1), e(3) }, { e(3) }}));
+
+            ctl.ground({{"multi", {}}});
+            test_solve(ctl.solve(), models);
+            REQUIRE(models == (ModelVec{{ a(2), b(1), e(3) }, { a(2), e(3) }, { b(1), e(3) }, { e(3) }}));
+
+        }
         SECTION("optimize") {
             ctl.add("base", {}, "2 {a; b; c; d}.\n"
                                 ":- a, b.\n"
@@ -225,7 +297,7 @@ TEST_CASE("solving", "[clingo]") {
             ctl.ground({{"base", {}}});
             SymbolVector model;
             CostVector optimum;
-            for (auto m : ctl.solve()) {
+            for (auto &m : ctl.solve()) {
                 model = m.symbols();
                 optimum = m.cost();
             }
@@ -247,7 +319,7 @@ TEST_CASE("solving", "[clingo]") {
             ctl.add("base", {}, "a. $x $= 1. #show b.");
             ctl.ground({{"base", {}}});
             bool sat = false;
-            for (auto m : ctl.solve()) {
+            for (auto &m : ctl.solve()) {
                 REQUIRE(m.type() == ModelType::StableModel);
                 REQUIRE(m.symbols(ShowType::Atoms) == (SymbolVector{Id("a")}));
                 REQUIRE(m.symbols(ShowType::Terms) == (SymbolVector{Id("b")}));
@@ -265,7 +337,7 @@ TEST_CASE("solving", "[clingo]") {
             ctl.add("base", {}, "1{a;b}1.");
             ctl.ground({{"base", {}}});
             int n = 0;
-            for (auto m : ctl.solve()) {
+            for (auto &m : ctl.solve()) {
                 REQUIRE(m.type() == ModelType::StableModel);
                 REQUIRE(m.thread_id() == 0);
                 ++n;
@@ -280,7 +352,7 @@ TEST_CASE("solving", "[clingo]") {
             ctl.add("base", {}, "a.");
             ctl.ground({{"base", {}}});
             int n = 0;
-            for (auto m : ctl.solve()) {
+            for (auto &m : ctl.solve()) {
                 REQUIRE(m.type() == ModelType::CautiousConsequences);
                 REQUIRE(m.thread_id() == 0);
                 ++n;
@@ -293,7 +365,7 @@ TEST_CASE("solving", "[clingo]") {
             ctl.add("base", {}, "{a}. #minimize { 1:a }.");
             ctl.ground({{"base", {}}});
             int n = 0;
-            for (auto m : ctl.solve()) {
+            for (auto &m : ctl.solve()) {
                 REQUIRE(m.type() == ModelType::StableModel);
                 REQUIRE(m.thread_id() == 0);
                 if (m.optimality_proven()) {
@@ -423,8 +495,8 @@ TEST_CASE("solving", "[clingo]") {
                     auto iter = ctl.solve();
                     {
                         MCB mcb(models);
-                        SECTION("c++") { for (auto m : iter) { mcb(m); }; }
-                        SECTION("java") { while (Model m = iter.next()) { mcb(m); }; }
+                        SECTION("c++") { for (auto &m : iter) { mcb(m); }; }
+                        SECTION("java") { while (auto &m = iter.next()) { mcb(m); }; }
                     }
                     REQUIRE(models == ModelVec{{Id("a")}});
                     REQUIRE(iter.get().is_satisfiable());
@@ -532,32 +604,32 @@ TEST_CASE("solving", "[clingo]") {
             REQUIRE(trail == std::vector<std::string>({"IP: incremental", "BS", "R: 1:-~1", "ES"}));
         }
         SECTION("events") {
-                ctl.add("base", {}, "{a}.");
-                ctl.ground({{"base", {}}});
-                int m = 0;
-                int f = 0;
-                bool goon = true;
-                SECTION("stop") { goon = false; }
-                SECTION("goon") { goon = true; }
-                struct EH : Clingo::SolveEventHandler {
-                    EH(bool goon, int &m, int &f) : goon(goon), m(m), f(f) { }
-                    bool on_model(Model const &) override {
-                        REQUIRE(f == 0);
-                        ++m;
-                        return goon;
-                    }
-                    void on_finish(SolveResult) override {
-                        ++f;
-                    }
-                    bool goon;
-                    int &m;
-                    int &f;
-                };
-                EH handler{goon, m, f};
-                auto handle = ctl.solve({}, &handler);
-                REQUIRE(test_solve(std::move(handle), models).is_satisfiable());
-                REQUIRE(m == (goon ? 2 : 1));
-                REQUIRE(f == 1);
+            ctl.add("base", {}, "{a}.");
+            ctl.ground({{"base", {}}});
+            int m = 0;
+            int f = 0;
+            bool goon = true;
+            SECTION("stop") { goon = false; }
+            SECTION("goon") { goon = true; }
+            struct EH : Clingo::SolveEventHandler {
+                EH(bool goon, int &m, int &f) : goon(goon), m(m), f(f) { }
+                bool on_model(Model &) override {
+                    REQUIRE(f == 0);
+                    ++m;
+                    return goon;
+                }
+                void on_finish(SolveResult) override {
+                    ++f;
+                }
+                bool goon;
+                int &m;
+                int &f;
+            };
+            EH handler{goon, m, f};
+            auto handle = ctl.solve(LiteralSpan{}, &handler);
+            REQUIRE(test_solve(std::move(handle), models).is_satisfiable());
+            REQUIRE(m == (goon ? 2 : 1));
+            REQUIRE(f == 1);
         }
     }
 }
