@@ -77,6 +77,10 @@ void Program::add(UStm &&stm) {
     }
 }
 
+void Program::addInput(Sig sig) {
+    sigs_.push(sig);
+}
+
 void Program::add(TheoryDef &&def, Logger &log) {
     auto it = theoryDefs_.find(def.name());
     if (it == theoryDefs_.end()) {
@@ -208,7 +212,31 @@ void Program::print(std::ostream &out) const {
     for (auto &x : stms_) { out << *x << "\n"; }
 }
 
-Ground::Program Program::toGround(DomainData &domains, Logger &log) {
+// Defines atoms that have been seen in earlier steps
+class DummyStatement : public Ground::Statement, private Ground::HeadOccurrence {
+public:
+    DummyStatement(UGTermVec terms, bool normal) : terms_{std::move(terms)}, normal_{normal}  {}
+    bool isNormal() const override { return normal_; }
+    void analyze(Dep::Node &node, Dep &dep) override {
+        for (auto &term : terms_) {
+            dep.provides(node, *this, get_clone(term));
+        }
+    }
+    void startLinearize(bool) override { }
+    void linearize(Context &, bool, Logger &) override { }
+    void enqueue(Ground::Queue &) override { }
+    void print(std::ostream &out) const override {
+        print_comma(out, terms_, ";", [](std::ostream &out, UGTerm const &term) { out << *term; });
+        out << ".";
+    }
+private:
+    void defines(IndexUpdater &, Ground::Instantiator *) override { }
+private:
+    UGTermVec terms_;
+    bool normal_;
+};
+
+Ground::Program Program::toGround(std::set<Sig> const &sigs, DomainData &domains, Logger &log) {
     HashSet<uint64_t> neg;
     Ground::Program::ClassicalNegationVec negate;
     auto gn = [&neg, &negate, &domains](Sig x) {
@@ -217,18 +245,22 @@ Ground::Program Program::toGround(DomainData &domains, Logger &log) {
         }
     };
     Ground::UStmVec stms;
+    if (!pheads.empty()) { stms.emplace_back(gringo_make_unique<DummyStatement>(std::move(pheads), true)); }
+    if (!nheads.empty()) { stms.emplace_back(gringo_make_unique<DummyStatement>(std::move(nheads), false)); }
     stms.emplace_back(make_locatable<Ground::ExternalRule>(Location("#external", 1, 1, "#external", 1, 1)));
     ToGroundArg arg(auxNames_, domains);
     Ground::SEdbVec edb;
     for (auto &block : blocks_) {
-        for (auto &x : block.edb->second) {
-            auto sig = x.sig();
-            if (sig.sign()) { gn(sig); }
-        }
-        edb.emplace_back(block.edb);
-        for (auto &x : block.stms) {
-            x->getNeg(gn);
-            x->toGround(arg, stms);
+        if (sigs.find(Sig{block.name.c_str() + 5, numeric_cast<uint32_t>(block.params.size()), false}) != sigs.end()) {
+            for (auto &x : block.edb->second) {
+                auto sig = x.sig();
+                if (sig.sign()) { gn(sig); }
+            }
+            edb.emplace_back(block.edb);
+            for (auto &x : block.stms) {
+                x->getNeg(gn);
+                x->toGround(arg, stms);
+            }
         }
     }
     for (auto &x : stms_) {
@@ -241,7 +273,10 @@ Ground::Program Program::toGround(DomainData &domains, Logger &log) {
         auto &node(dep.add(std::move(x), normal));
         node.stm->analyze(node, dep);
     }
-    Ground::Program prg(std::move(edb), dep.analyze(), std::move(negate));
+    auto ret = dep.analyze();
+    Ground::Program prg(std::move(edb), std::move(std::get<0>(ret)), std::move(negate));
+    pheads = std::move(std::get<1>(ret));
+    nheads = std::move(std::get<2>(ret));
     for (auto &sig : sigs_) {
         domains.add(sig);
     }
