@@ -123,6 +123,7 @@ public:
     virtual void define(Location const &loc, String name, TermUid value, bool defaultDef, Logger &log) override;
     virtual void optimize(Location const &loc, TermUid weight, TermUid priority, TermVecUid cond, BdLitVecUid body) override;
     virtual void showsig(Location const &loc, Sig sig, bool csp) override;
+    virtual void defined(Location const &loc, Sig sig) override;
     virtual void show(Location const &loc, TermUid t, BdLitVecUid body, bool csp) override;
     virtual void python(Location const &loc, String code) override;
     virtual void lua(Location const &loc, String code) override;
@@ -621,6 +622,11 @@ void TestNongroundProgramBuilder::showsig(Location const &, Sig sig, bool csp) {
     statements_.emplace_back(str());
 }
 
+void TestNongroundProgramBuilder::defined(Location const &, Sig sig) {
+    current_ << "#defined " << sig << ".";
+    statements_.emplace_back(str());
+}
+
 void TestNongroundProgramBuilder::show(Location const &, TermUid t, BdLitVecUid bodyuid, bool csp) {
     current_ << "#show " << (csp ? "$" : "") << terms_.erase(t);
     StringVec body(bodies_.erase(bodyuid));
@@ -1208,6 +1214,10 @@ TEST_CASE("input-nongroundprogrambuilder", "[input]") {
         REQUIRE("#program base().\n#show X:p(X);1<=#count{q(X):p(X)}." == parse("#show X:p(X), 1 { q(X):p(X) }."));
     }
 
+    SECTION("input") {
+        REQUIRE("#program base().\n#defined p/1." == parse("#defined p/1."));
+    }
+
     SECTION("include") {
         struct Del {
             Del()  { std::ofstream("test_include.lp") << "b.\n"; }
@@ -1250,7 +1260,9 @@ TEST_CASE("input-nongroundprogrambuilder", "[input]") {
     SECTION("theory") {
         // NOTE: things would be less error prone if : and ; would not be valid theory connectives
         REQUIRE("#program base().\n&x{}." == parse("&x { }."));
+        REQUIRE("#program base().\n&x{}." == parse("&x."));
         REQUIRE("#program base().\n#false:-&x{}." == parse(":-&x { }."));
+        REQUIRE("#program base().\n#false:-&x{}." == parse(":-&x."));
         REQUIRE("#program base().\n&x{} < 42." == parse("&x { } < 42."));
         REQUIRE("#program base().\n#false:-&x{} < 42." == parse(":-&x { } < 42."));
         REQUIRE("#program base().\n#false:-&x{} < 42 + 17 ^ (- 1)." == parse(":-&x { } < 42+17^(-1)."));
